#!/usr/bin/env python
import time, FFT, FFT2, FFTW
from MLab import rand, array
def timeit(n, fft, ax):
    a = rand(128,128)
    start=time.time()
    for i in range(n):
        b=fft(a, 128, ax)
    return time.time()-start

def timeit_ip(n, fft, ax):
    a = rand(128,128)
    start=time.time()
    for i in range(n):
        b = array(a, 'D')
        fft(b, 128, ax)
    return time.time()-start

def timeit2(n, fft, ax):
    a = rand(30, 30, 30)
    start=time.time()
    for i in range(n):
        b=fft(a, axes=ax)
    return time.time()-start

def timeit2_ip(n, fft, ax):
    a = rand(30, 30, 30)
    start=time.time()
    for i in range(n):
        b = array(a, 'D')
        fft(b, axes=ax)
    return time.time()-start

def timeit3(n, fft, ax):
    a = rand(40, 40, 40)
    start=time.time()
    for i in range(n):
        b=fft(a, axes=ax)
    return time.time()-start

def timeit3_ip(n, fft, ax):
    a = rand(40, 40, 40)
    start=time.time()
    for i in range(n):
        b = array(a, 'D')
        fft(b, axes=ax)
    return time.time()-start

print "*************************************"
print "*************  Benchmark  ***********"
print "*************************************"
print "\n===== 1-D comparison ====="
fft1=FFT.fft
fft2=FFTW.fft
fft3=FFTW.cost
fft4=FFTW.fft_ip

print "\nfor an array 128x128, transform on axis=1"
dif1 = timeit(10, fft1, 1)
print "FFT1: " + repr(round(dif1,4)) + " seconds"
dif2 = timeit(10, fft2, 1)
print "FFT2: " + repr(round(dif2,4)) + " seconds"
dif3 = timeit(10, fft3, 1)
print "FFT3: " + repr(round(dif3,4)) + " seconds"
dif4 = timeit_ip(10, fft4, 1)
print "FFT4: " + repr(round(dif4,4)) + " seconds"

print "\nfor an array 128x128, transform on axis=0"
dif1 = timeit(10, fft1, 0)
print "FFT1: " + repr(round(dif1,4)) + " seconds"
dif2 = timeit(10, fft2, 0)
print "FFT2: " + repr(round(dif2,4)) + " seconds"
dif3 = timeit(10, fft3, 0)
print "FFT3: " + repr(round(dif3,4)) + " seconds"
dif4 = timeit_ip(10, fft4, 0)
print "FFT4: " + repr(round(dif4,4)) + " seconds"


print "\n=====2-D comparison====="
fft1=FFT.fft2d
fft2=FFTW.fftnd
fft3=FFTW.rfftnd
fft4=FFTW.fftnd_ip

print "\nfor an array 30x30x30, transform on axis=(0,1)"
dif1 = timeit2(10, fft1, (0,1))
print "FFT1: " + repr(round(dif1,4)) + " seconds"
dif2 = timeit2(10, fft2, (0,1))
print "FFT2: " + repr(round(dif2,4)) + " seconds"
dif3 = timeit2(10, fft3, (0,1))
print "FFT3: " + repr(round(dif3,4)) + " seconds"
dif4 = timeit2_ip(10, fft4, (0,1))
print "FFT4: " + repr(round(dif4,4)) + " seconds"

print "\nfor an array 30x30x30, transform on axis=(1,2)"
dif1 = timeit2(10, fft1, (1,2))
print "FFT1: " + repr(round(dif1,4)) + " seconds"
dif2 = timeit2(10, fft2, (1,2))
print "FFT2: " + repr(round(dif2,4)) + " seconds"
dif3 = timeit2(10, fft3, (1,2))
print "FFT3: " + repr(round(dif3,4)) + " seconds"
dif4 = timeit2_ip(10, fft4, (1,2))
print "FFT4: " + repr(round(dif4,4)) + " seconds"

"""
print "\nfor an array 30x30x30, transform on axis=(0,2)"
dif1 = timeit2(10, fft1, (0,2))
print "FFT1: " + repr(round(dif1,4)) + " seconds"
dif2 = timeit2(10, fft2, (0,2))
print "FFT2: " + repr(round(dif2,4)) + " seconds"
dif3 = timeit2(10, fft3, (0,2))
print "FFT3: " + repr(round(dif3,4)) + " seconds"
dif4 = timeit2_ip(10, fft4, (0,2))
print "FFT4: " + repr(round(dif4,4)) + " seconds"

print "\nfor an array 40x40x40, transform on axis=(0,2)"
dif1 = timeit3(10, fft1, (0,2))
print "FFT1: " + repr(round(dif1,4)) + " seconds"
dif2 = timeit3(10, fft2, (0,2))
print "FFT2: " + repr(round(dif2,4)) + " seconds"
dif3 = timeit3(10, fft3, (0,2))
print "FFT3: " + repr(round(dif3,4)) + " seconds"
dif4 = timeit3_ip(10, fft4, (0,2))
print "FFT4: " + repr(round(dif4,4)) + " seconds"
"""
